/*  Copyright (C) 2020 Andrea G. Monaco
 *
 *  This file is part of libre-sapienza.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */


#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <termios.h>

#include <libintl.h>
#define _(String) gettext (String)

#include "input.h"


/* read password with disabled echo */
const char *
read_password (void)
{
  char *pass = NULL;
  size_t pass_l = 0;
  int r;
  struct termios t_old, t_new;
  
  tcgetattr (fileno (stdin), &t_old);
  t_new = t_old;
  t_new.c_lflag &= ~ECHO;
  tcsetattr (fileno (stdin), TCSAFLUSH, &t_new);

  /* maximum password size allowed by the server is 16 */
  pass = malloc (32);
  if (!pass)
    {
      fprintf (stderr, _("memory exhausted!\n"));
      exit (1);
    }
  pass_l = 32;
  
  printf (_("password: "));
  r = getline (&pass, &pass_l, stdin);
  if (r == -1)
    {
      fprintf (stderr, _("input error\n"));
      exit (1);
    }
  
  /* getline also returns the newline, so discard it */
  pass [r-1] = 0;
  putchar ('\n');
  tcsetattr (fileno (stdin), TCSAFLUSH, &t_old);

  return pass;
}


/* read an integer from the keyboard in a robust way, using getline and then converting with strtol.
   takes as an argument the estimated size of input, to preallocate the array */
int
read_int (int est)
{
  char *buff;
  size_t buff_l;
  int ret;

  /* preallocate the likely (maximum) dimension in digit of the number */
  buff = malloc (est);
  
  if (!buff)
    {
      fprintf (stderr, _("memory exhausted!\n"));
      exit (1);
    }
  
  buff_l = est;
 
  if (getline (&buff, &buff_l, stdin) == -1)
    {
      fprintf (stderr, _("input error\n"));
      exit (1);
    }

  ret = strtol (buff, NULL, 10);
  free (buff);
  return ret;
}


/* print prompt and read user action */
int
prompt_and_read_user_action (void)
{
  printf (_("\n***\n"
	    "1 - print available exams\n"
	    "2 - print completed exams\n"
	    "3 - print active reservations\n"
	    "4 - print student info\n"
	    "5 - print paid taxes\n"
	    "6 - print unpaid taxes\n"
	    "7 - exit\n\n"
	    "> "));
  return read_int (8);
}
